<script setup lang="ts">
import { computed, onMounted, ref } from 'vue';

import { useVbenDrawer, useVbenModal } from '@vben/common-ui';
import { DictEnum } from '@vben/constants';
import { $t } from '@vben/locales';
import { addFullName, cloneDeep, getPopupContainer, listToTree } from '@vben/utils';

import { InputSearch } from 'ant-design-vue';

import { useVbenForm } from '#/adapter/form';
import { listEquType } from '#/api/eims/equ-type';
import { addRepairReq, getRepairReq, updateRepairReq } from '#/api/eims/repair-req';
import { getDeptTree, userList } from '#/api/system/user';
import { getDictOptions } from '#/utils/dict';
import CodeInput from '#/views/eims/components/code-input.vue';
import equModal from '#/views/eims/components/equ-modal.vue';

import { drawerSchema } from './data';

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    formItemClass: 'col-span-2',
    componentProps: {
      class: 'w-full'
    },
    labelWidth: 120
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }
    // 动态切换表单
    formApi.updateSchema([
      {
        componentProps: () => ({
          disabled: isUpdate.value,
          options: getDictOptions(DictEnum.REPAIR_REQ_TYPE),
          async onSelect(reqType: string) {
            // 类型见字典-REPAIR_REQ_TYPE
            formApi.updateSchema([
              {
                dependencies: {
                  show: () => reqType === '1',
                  triggerFields: ['']
                },
                fieldName: 'equName'
              }
            ]);
          }
        }),
        fieldName: 'reqType'
      },
      {
        dependencies: {
          show: () => false,
          triggerFields: ['']
        },
        fieldName: 'equName'
      }
    ]);
    drawerApi.drawerLoading(true);
    const { id } = drawerApi.getData() as { id?: number | string };
    isUpdate.value = !!id;
    // 初始化
    await setupDeptSelect();
    await setupEquTypeSelect();
    // 更新 && 赋值
    if (isUpdate.value && id) {
      const record = await getRepairReq(id);
      const reqType = record.reqType;
      await formApi.setValues(record);
      // 更新时不可修改盘点范围
      if (isUpdate.value && record.reqDept) {
        await setupUserOptions(record.reqDept);
      }
      formApi.updateSchema([
        {
          dependencies: {
            show: () => reqType === '1',
            triggerFields: ['']
          },
          fieldName: 'equName'
        }
      ]);
    }


    drawerApi.drawerLoading(false);
  }
});

const [EquModal, equModalApi] = useVbenModal({
  connectedComponent: equModal,
  draggable: true,
  title: '选择设备'
});

function handleOpenModal() {
  equModalApi.setData({});
  equModalApi.open();
}

/**
 * 加载设备类树
 */
const equTypeTree = ref();

async function setupEquTypeSelect() {
  const result = (await listEquType()) || [];
  // 构建组件需要数据
  const equArray = result.map((item) => {
    return {
      ...item, // 保留原有对象的所有属性
      id: item.equTypeId,
      label: item.typeName,
      weight: item.orderNum
    };
  });
  // support i18n
  equArray.forEach((item) => {
    item.typeName = $t(item.typeName);
  });
  const equTree = listToTree(equArray, { id: 'id', pid: 'parentId' });
  equTypeTree.value = equTree;
}

/**
 * 用户的加载
 */
async function setupUserOptions(deptId: any) {
  const params = { deptId };
  const userPageResult = await userList({
    pageNum: 1,
    pageSize: 500,
    ...params
  });
  const options = userPageResult.rows.map((item) => ({
    label: item.nickName || item.userName,
    value: item.userId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该部门下暂无用户';
  formApi.updateSchema([
    {
      componentProps: { options, placeholder, filterOption },
      fieldName: 'reqUser'
    }
  ]);
}

/**
 * 初始化部门选择
 */
async function setupDeptSelect() {
  // updateSchema
  const deptTree = await getDeptTree();
  // 选中后显示在输入框的值 即父节点 / 子节点
  addFullName(deptTree, 'label', ' / ');
  formApi.updateSchema([
    {
      componentProps: (formModel) => ({
        class: 'w-full',
        fieldNames: {
          key: 'id',
          value: 'id',
          children: 'children'
        },
        getPopupContainer,
        async onSelect(deptId: number | string) {
          /** 根据部门ID加载用户 */
          await setupUserOptions(deptId);
          /** 变化后需要重新选择用户 */
          formModel.reqUser = undefined;
        },
        placeholder: '请选择',
        showSearch: true,
        treeData: deptTree,
        treeDefaultExpandAll: true,
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'label',
        // 选中后显示在输入框的值
        treeNodeLabelProp: 'fullName'
      }),
      fieldName: 'reqDept'
    }
  ]);
}

async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    const data = cloneDeep(await formApi.getValues());
    await (isUpdate.value ? updateRepairReq(data) : addRepairReq(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}

/**
 * 打开查询设备
 */
function onSearchEqu() {
  handleOpenModal();
}

/**
 * 更新选择的设备
 * @param equ
 */
async function updateEqu(equ: any) {
  await formApi.setValues({ 'equId': equ.equId, 'equName': equ.equName });
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm>
      <template #code="slotProps">
        <CodeInput v-bind="slotProps" :disabled="isUpdate" prefix="BXD" />
      </template>
      <template #equName="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择设备" @search="onSearchEqu" v-bind="slotProps"
                     :disabled="isUpdate" />
      </template>
    </BasicForm>
    <EquModal class="w-[1200px]" @update-equ="updateEqu" />
  </BasicDrawer>
</template>
